-- $Rev: 39 $

local addonName, L = ...;
local BH_ProjectUrl = "http://www.curse.com/addons/wow/byehakmud";
local BH_IsLoaded = false;

--- Properties of Bye Hakmud

function BH_SetByeHakmundEnabled(enabled)
	local oldEnabled = BH_IsByeHakmudEnabled();
	BH_IsDismountHakmudEnabled = enabled;
	if (oldEnabled ~= enabled) then
		BH_SetListen4Vehicle(BH_IsByeHakmudEnabled());
		if (BH_IsByeHakmudEnabled()) then
			BH_PostMessage (L["ByeHakmud turned on: Vendor can be ejected when the mount is invoked"]);
		else
			BH_PostMessage (L["ByeHakmud turned off: Vendor stays on the mount"]);
		end
	end
end

function BH_IsHakmudKeptOnShift()
	local retVal = true;
	if (BH_HakmudKeptOnShift ~= nil) then
		retVal = BH_HakmudKeptOnShift;
	end
	return retVal;
end

function BH_SetHakmudKeptOnShift(keep)
	BH_HakmudKeptOnShift = keep;
end

function BH_IsOnlyEjectWhenInParty()
    local retVal = false;
    if (BH_OnlyEjectWhenInParty ~= nil) then
	retVal = BH_OnlyEjectWhenInParty;
    end
    return retVal;
end

function BH_SetOnlyEjectWhenInParty(EjectInParty)
    BH_OnlyEjectWhenInParty = EjectInParty
end

function BH_GetChannelToNotify()
    local retVal = "local"
    if (BH_ChannelToNotify ~= nil) then
	retVal = BH_ChannelToNotify
    end
    return retVal
end

function BH_SetChannelToNotify(channel)
    BH_ChannelToNotify = channel
end

function BH_GetNotifyMsg()
    local retVal = L["ByeHakmud ejected vendor from the mount"];
    if BH_NotifyMsg ~= nil then
	retVal = BH_NotifyMsg
    end
    return retVal
end

function BH_SetNotifyMsg(msg)
    BH_NotifyMsg = msg
end

function BH_GetVersion()
	local version = "unversioned";
	local repoDate = "$Date: 2012-10-17 00:53:53 +0000 (Wed, 17 Oct 2012) $";
	local repoUrl = "$URL: file:///media/cf-repositories/svn/wow/byehakmud/mainline/tags/V%201.03.01/ByeHakmud.lua $";

	repoUrl = string.gsub(repoUrl, "%%20", " "); -- spaces are url encoded
	local tagPattern = "/tags/[^/]*";
	local tagStart, tagEnd = string.find(repoUrl, tagPattern);

	if (tagStart ~= nil and tagEnd ~= nil) then
		tagStart = tagStart + string.len("/tags/");
		version = string.sub(repoUrl, tagStart, tagEnd);
	else
		local datePattern = "$Date:[^$]*";
		local dateStart, dateEnd = string.find(repoDate, datePattern);
		if (dateStart ~= nil and dateEnd ~= nil) then
			dateStart = dateStart + string.len("$Date:");
			dateStr = string.sub(repoDate, dateStart, dateEnd);
			version = "unversioned "..tostring(dateStr);
		end
	end
	
	return version;
end 

 --- Config Panel View Accessors
 local BH_ConfigPanel;
 
 function BH_GetConfigPanel()
	return BH_ConfigPanel;
end

function BH_GetWidgetByName(name)
    local panel = BH_GetConfigPanel();
    return _G[panel:GetName()..name];
end
 
function BH_GetBHEnabledCB()
    return BH_GetWidgetByName("_CB_ByeHakmudEnabled");
end

function BH_GetBHOnlyEjectWhenInPartyCB()
    return BH_GetWidgetByName("_CB_OnlyEjectWhenInGroup");
end

function BH_GetBHChannelToNotifyCB()
    return BH_GetWidgetByName("_CB_CHANNEL_TO_NOTIFY");
end

function BH_GetBHNotifyMsgEB()
    return BH_GetWidgetByName("_EB_NOTIFY_MSG");
end

--- Config Panel View Initialization

function BH_AddConfigPanelToAddonOptions()
	local panel = BH_GetConfigPanel();
	panel.name = "Bye Hakmud";
	panel.okay = function (self) BH_OnOk(); end;
	panel.refresh = function (self) BH_OnRefresh(); end;
	InterfaceOptions_AddCategory(panel);
end

function TL(s)
  return L[s];
end

--- Config Panel Controller

function BH_OnOk()
	local enabled = BH_GetBHEnabledCB():GetChecked() == 1;
	BH_SetByeHakmundEnabled(enabled);

	local onlyEjectWhenInParty = BH_GetBHOnlyEjectWhenInPartyCB():GetChecked() == 1;
	BH_SetOnlyEjectWhenInParty(onlyEjectWhenInParty);

	BH_SetHakmudKeptOnShift(BH_RB_Model_IsKeptOnShift());
	
	local channelToNotify = UIDropDownMenu_GetSelectedValue(BH_GetBHChannelToNotifyCB());
	BH_SetChannelToNotify(channelToNotify)
	
	local notifyMsg = BH_GetBHNotifyMsgEB():GetText()
	BH_SetNotifyMsg(notifyMsg)
	
	BH_ApplySeatCheckBoxes()
end

function BH_OnRefresh()
	BH_GetBHEnabledCB():SetChecked(BH_IsByeHakmudEnabled());
	BH_GetBHOnlyEjectWhenInPartyCB():SetChecked(BH_IsOnlyEjectWhenInParty());
	BH_RB_Model_SetKeptOnShift(BH_IsHakmudKeptOnShift());
end

function BH_ConfigPanel_OnLoad(frame)
	BH_ConfigPanel = frame;
	BH_AddConfigPanelToAddonOptions();

	local subFrame = _G[frame:GetName().."SubText"]
	if subFrame then 
	    subFrame:SetText(TL(
		"ByeHakmud can automatically dismount a vendor when "..
		"a mount like the Traveler's Tundra Mammoth is invoked. "..
		"That can be useful if you want to keep a seat free for a "..
		"group member to join your mount and repair his armor while riding."
	      ));
	end

	subFrame = _G[frame:GetName().."VersionText"]
	if subFrame then
	    subFrame:SetText(BH_GetVersion() .."\n" .. BH_ProjectUrl);
	end

	frame:RegisterEvent("ADDON_LOADED");
end

---  

function BH_IsByeHakmudEnabled()
	local retVal = true;
	if (BH_IsDismountHakmudEnabled ~= nil) then
		retVal = BH_IsDismountHakmudEnabled;
	end
	return retVal;
end

--- Controller of Radio Button 'Hakmud kept on <shift>'
local BH_RB_Model_KeptOnShift;

function BH_RB_Model_IsKeptOnShift()
	return BH_RB_Model_KeptOnShift;
end

function BH_RB_Model_SetKeptOnShift(keep)
	BH_RB_Model_KeptOnShift = keep;
	BH_UpdateHakmudKeptOnShiftButtons();
end

function BH_UpdateHakmudKeptOnShiftButtons()
	local keptOnShift = BH_RB_Model_IsKeptOnShift();
	_G["ByeHakmudConfigFrame_RB_SHIFT_KEEPS"]:SetChecked(keptOnShift);
	_G["ByeHakmudConfigFrame_RB_SHIFT_KICKS"]:SetChecked(not keptOnShift);
end

---
function BH_DropDownCallback(info)
    UIDropDownMenu_SetSelectedValue (info.owner, info.value)
end

local entry = {}
function BH_CB_AddEntry(owner, text, value, func)
    entry.owner	= owner
    entry.text	= text
    entry.value	= value
    entry.checked = false
    entry.func = func
    UIDropDownMenu_AddButton(entry)
end

function BH_CB_ChannelToNotifyInitialize(self)
    BH_CB_AddEntry(self, L["don't post"], "none", BH_DropDownCallback)
    BH_CB_AddEntry(self, L["local"], "local", BH_DropDownCallback)
    BH_CB_AddEntry(self, L["say"], "say", BH_DropDownCallback)
    BH_CB_AddEntry(self, L["auto party/raid/battleground"], "auto", BH_DropDownCallback)
    BH_CB_AddEntry(self, L["party only"], "party", BH_DropDownCallback)
    BH_CB_AddEntry(self, L["raid only"], "raid", BH_DropDownCallback)
    BH_CB_AddEntry(self, L["battleground only"], "battleground", BH_DropDownCallback)
    BH_CB_AddEntry(self, L["yell"], "yell", BH_DropDownCallback)
end

function BH_CB_ChannelToNotifyOnLoad(self)
    UIDropDownMenu_Initialize(self, BH_CB_ChannelToNotifyInitialize)
    local selVal = BH_GetChannelToNotify()
    UIDropDownMenu_SetSelectedValue(self, selVal)
end

--- Notify Msg Edit Box
function BH_EB_NotifyMSGOnShow(self)
    self:SetText("")
    self:SetText(BH_GetNotifyMsg())
end

--- Mount Selector
 
function BH_ApplySeatCheckBoxes()
    local mountSelector = BH_GetWidgetByName("_CB_MountSelector");
    local creatureID = UIDropDownMenu_GetSelectedValue(mountSelector);
    if creatureID then
	local seatsToEject = {}	    
	local seatsToEjectIdx = 1
	local checkBoxID = 1
	local checkBox = nul
	repeat
	    local checkBox = BH_GetWidgetByName("_CB_EjectVendor"..checkBoxID)
	    if checkBox then
		local checked = checkBox:GetChecked()
		if checked then
		    seatsToEject[seatsToEjectIdx] = checkBoxID
		    seatsToEjectIdx = seatsToEjectIdx + 1
		end
		checkBoxID = checkBoxID + 1
	    end
	until checkBox == nil
	BH_SetSeatsToEject4Mount(creatureID, seatsToEject)
    end
end

function BH_UpdateSeatCheckBoxes(creatureID)
    local seatsToEject = BH_GetSeatsToEject4Mount(creatureID);
    
    local checkBoxID = 1
    local checkBox = nul
    repeat
	local checkBox = BH_GetWidgetByName("_CB_EjectVendor"..checkBoxID)
	if checkBox then
	    local checked = false
	    for i, seatID in ipairs(seatsToEject) do
		if seatID == checkBoxID then
		    checked = true
		end
	    end
	    checkBox:SetChecked(checked)
	    checkBoxID = checkBoxID + 1
	end
    until checkBox == nil
end

function BH_CB_MountSelectorEntryCallback(info)
    BH_ApplySeatCheckBoxes()
    BH_DropDownCallback(info)
    BH_UpdateSeatCheckBoxes(info.value)
end

function BH_CB_MountSelectorInitialize(self)
    local companionCount = GetNumCompanions("MOUNT");
    for i = 1, companionCount do
	local creatureID, creatureName, creatureSpellID, icon, issummoned = GetCompanionInfo("MOUNT", i);
	local seatsToEject = BH_GetSeatsToEject4Mount(creatureID);
	if seatsToEject then
	    BH_CB_AddEntry(self, creatureName, creatureID, BH_CB_MountSelectorEntryCallback)
	end
    end
end

function BH_CB_MountSelectorOnShow(self)
    UIDropDownMenu_Initialize(self, BH_CB_MountSelectorInitialize)
    if BH_IsLoaded then
	local selectedCreatureID = UIDropDownMenu_GetSelectedValue(self);
	
	local companionCount = GetNumCompanions("MOUNT");
	for i = 1, companionCount do
	    local creatureID, creatureName, creatureSpellID, icon, issummoned = GetCompanionInfo("MOUNT", i);
	    local seatsToEject = BH_GetSeatsToEject4Mount(creatureID);
	    if seatsToEject then
		if issummoned or selectedCreatureID == nil then
		    selectedCreatureID = creatureID
		end
	    end
	end
	
	UIDropDownMenu_SetSelectedValue(self, selectedCreatureID)
	BH_UpdateSeatCheckBoxes(selectedCreatureID)
    end
end

--- Bye Hakmud Engine

function BH_SetListen4Event(enabled, eventName)
	if (enabled) then
		if (not BH_GetConfigPanel():IsEventRegistered(eventName)) then
			BH_GetConfigPanel():RegisterEvent(eventName); 
		end
	else 
		if (BH_GetConfigPanel():IsEventRegistered(eventName)) then
			BH_GetConfigPanel():UnregisterEvent(eventName); 
		end
	end
end

function BH_SetListen4Vehicle(enabled)
	BH_SetListen4Event(enabled, "PLAYER_GAINS_VEHICLE_DATA");
end

function BH_SetListen4Passenger(enabled)
	BH_SetListen4Event(enabled, "VEHICLE_PASSENGERS_CHANGED");
end

function BH_IsPartyAllowsEject()
	local retVal = true;
	if BH_IsOnlyEjectWhenInParty() then
	    retVal = UnitInParty("player")
	end
	return retVal;
end

function BH_GetSeatsToEject4Mount(creatureID)
    local retVal = nil;
    if BH_MountIDToSeat then
	retVal = BH_MountIDToSeat[creatureID]
    end
    
    if retVal == nil then
	retVal = BH_MountIDToSeatDefaultList[creatureID]
    end
    
    return retVal;
end

function BH_SetSeatsToEject4Mount(creatureID, seatsToEject)
    if BH_MountIDToSeat == nil then
       BH_MountIDToSeat = {}
    end
    BH_MountIDToSeat[creatureID] = seatsToEject
end

function BH_EjectHakmud()
	function BH_GetSeatsToEject()
		local retVal = nil;
		local companionCount = GetNumCompanions("MOUNT");
		for i = 1, companionCount do
			local creatureID, creatureName, creatureSpellID, icon, issummoned = GetCompanionInfo("MOUNT", i);
			if issummoned then
			    retVal = BH_GetSeatsToEject4Mount(creatureID);
			    break;
			end
		end
		return retVal;
	end

	local keepHakmud = IsShiftKeyDown();
	if (not BH_IsHakmudKeptOnShift()) then
		keepHakmud = not keepHakmud;
	end
	
	if ((not keepHakmud)
	    and BH_IsPartyAllowsEject())
	then 
	    BH_SetListen4Passenger(false);
	    local seatsToEject = BH_GetSeatsToEject();
	    if seatsToEject then
		local ejected = false
		for i, seat in ipairs(seatsToEject) do
		    if CanEjectPassengerFromSeat(seat) then
			EjectPassengerFromSeat(seat)
			ejected = true
		    end
		end
		
		if (ejected) then
		    BH_PostMessage(BH_GetNotifyMsg(), BH_GetChannelToNotify())
		end
	    end
	end
end

--- Bye Hakmud Engines Event Handlers

function BH_OnAddonLoaded(addonName)
	if (addonName == "ByeHakmud") then
		BH_SetListen4Vehicle(BH_IsByeHakmudEnabled());
		BH_GetConfigPanel():UnregisterEvent("ADDON_LOADED");
		BH_IsLoaded = true;
	end
end

function BH_OnVehiclePassengersChanged()
	BH_EjectHakmud() ;
end

function BH_OnPlayerGainsVehicleData()
	BH_SetListen4Passenger(true);
end

function BH_OnEvent(self, event, arg1, ...)
  if      (event == "ADDON_LOADED") then BH_OnAddonLoaded(arg1);
  elseif (event == "VEHICLE_PASSENGERS_CHANGED") then BH_OnVehiclePassengersChanged();
  elseif (event == "PLAYER_GAINS_VEHICLE_DATA") then BH_OnPlayerGainsVehicleData();
  end
end

--- Command line interface

SLASH_BYEHAKMUD1 = '/ByeHakmud';
function SlashCmdList.BYEHAKMUD(msg, editbox)
	if msg == 'on' then
		BH_SetByeHakmundEnabled(true);
	elseif msg == 'off' then
		BH_SetByeHakmundEnabled(false);
	elseif msg == 'toggle' then
		BH_SetByeHakmundEnabled(not BH_IsByeHakmudEnabled());
	elseif msg == 'keep' then
		BH_SetHakmudKeptOnShift(true);
	elseif msg == 'kick' then
		BH_SetHakmudKeptOnShift(false);
	elseif msg == 'gui' then
		InterfaceOptionsFrame_OpenToCategory(ByeHakmudConfigFrame);
	else
		BH_PostMessage(
			"ByeHakmud Version: " .. BH_GetVersion() .. " " .. BH_ProjectUrl .."\n" ..
			L["/ByeHakmud on -- automatic ejection enabled\n" ..
			  "/ByeHakmud off -- automatic ejection turned off\n"..
			  "/ByeHakmud toggle -- switch automatich ejection on / off\n"..
			  "/ByeHakmud keep -- holding <shift> keeps Hakmud on the mount\n"..
			  "/ByeHakmud kick -- holding <shift> ejects Hakmud from the mount\n"..
			  "/ByeHakmud gui -- show configuration panel"
			]
		);
	end
end

function BH_PostMessage(msg, channel)
    if channel== nil then channel = "local" end
    
    if channel == "local" then
	print(msg);
    else
	local chatType = nil
	
	if channel == "say" then
	    chatType = "SAY"
	elseif channel == "yell" then
	    chatType = "YELL"
	elseif UnitInParty("player") then
	    if channel == "party" then
		chatType = "PARTY"
	    elseif channel == "raid" then
		chatType = "RAID"
	    elseif channel == "battleground" then
		chatType = "BATTLEGROUND"
	    elseif channel == "auto" then
		if UnitInParty("player") then
		    chatType = "PARTY"
		    if UnitInBattleground("player") then
			chatType = "BATTEGROUND"
		    elseif UnitInRaid("player") then
			chatType = "RAID"
		    end
		end
	    end
	end
	
	if chatType then
	    SendChatMessage(msg, chatType)
	end
    end
end

function BH_ShowFrameProps(frame)
    print(tostring(tostring(frame:GetName()))
	.." Width: "..tostring(frame:GetWidth())
	.." Height: "..tostring(frame:GetHeight())
	.." Left: "..tostring(frame:GetLeft())
	.." Top: "..tostring(frame:GetTop())
	.." Right: "..tostring(frame:GetRight())
	.." Bottom: "..tostring(frame:GetBottom())
    )

    local anchorPointCount = frame:GetNumPoints()
    if anchorPointCount and anchorPointCount > 0 then 
	for pointIdx = 1, frame:GetNumPoints() do
	    local point, relativeTo, relativePoint, xOffset, yOffset = frame:GetPoint(pointIdx)
	    local relativeToName
	    if relativeTo then
		relativeToName = relativeTo:GetName()
	    end
	    print("point:"..tostring(point)
		.." relativeTo:"..tostring(relativeToName)
		.." relativePoint:"..tostring(relativePoint)
		.." xOffset:"..tostring(xOffset)
		.." yOffset:"..tostring(yOffset) 
	    )
	end
    else
	print("no anchor points: "..tostring(achorPointCount))
    end
end